from future import standard_library
standard_library.install_aliases()
import logging
logger = logging.getLogger('Prismedia')
logger.setLevel(logging.INFO)
ch = logging.StreamHandler()
ch.setLevel(logging.INFO)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s: %(message)s')
ch.setFormatter(formatter)
logger.addHandler(ch)

from . import upload



from os.path import join, abspath, isfile, dirname, exists
from os import listdir
from shutil import copyfile
import logging
logger = logging.getLogger('Prismedia')

from . import utils


def genconfig():
    path = join(dirname(__file__), 'config')
    files = [f for f in listdir(path) if isfile(join(path, f))]

    for f in files:
        final_f = f.replace(".sample", "")
        if exists(final_f) and not utils.ask_overwrite(final_f + " already exists. Do you want to overwrite it?"):
            continue

        copyfile(join(path, f), final_f)
        logger.info(str(final_f) + " correctly generated, you may now edit it to fill your credentials.")


if __name__ == '__main__':
    genconfig()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['prismedia']

package_data = \
{'': ['*'],
 'prismedia': ['config/*',
               'samples/cli_nfo.txt',
               'samples/cli_nfo.txt',
               'samples/cli_nfo.txt',
               'samples/cli_nfo.txt',
               'samples/cli_nfo.txt',
               'samples/full_nfo_examples.txt',
               'samples/full_nfo_examples.txt',
               'samples/full_nfo_examples.txt',
               'samples/full_nfo_examples.txt',
               'samples/full_nfo_examples.txt',
               'samples/nfo.txt',
               'samples/nfo.txt',
               'samples/nfo.txt',
               'samples/nfo.txt',
               'samples/nfo.txt',
               'samples/samples.txt',
               'samples/samples.txt',
               'samples/samples.txt',
               'samples/samples.txt',
               'samples/samples.txt',
               'samples/yourvideo.txt',
               'samples/yourvideo.txt',
               'samples/yourvideo.txt',
               'samples/yourvideo.txt',
               'samples/yourvideo.txt']}

install_requires = \
['Unidecode>=1.0.23',
 'clint>=0.5.1',
 'configparser>=3.7.1',
 'docopt>=0.6.2',
 'future>=0.17.1',
 'google-api-python-client>=1.7.6',
 'google-auth-httplib2>=0.0.3',
 'google-auth-oauthlib>=0.2.0',
 'google-auth>=1.6.1',
 'httplib2>=0.12.1',
 'oauthlib==2.1.0',
 'pytz==2022.1',
 'requests-oauthlib==1.1.0',
 'requests-toolbelt>=0.9.1',
 'requests>=2.18.4',
 'schema>=0.7.1',
 'tzlocal>=1.5.1',
 'uritemplate>=3.0.0',
 'urllib3>=1.22']

entry_points = \
{'console_scripts': ['prismedia = prismedia.upload:main',
                     'prismedia-init = prismedia.genconfig:genconfig']}

setup_kwargs = {
    'name': 'prismedia',
    'version': '0.12.2',
    'description': 'scripting your way to upload videos on peertube and youtube',
    'long_description': '# Prismedia\n\nScripting your way to upload videos to peertube and youtube. Works with Python 3.5+.\n\n[TOC]: #\n\n## Table of Contents\n- [Installation](#installation-and-upgrade)\n  - [From pip](#from-pip)\n  - [From source](#from-source)\n- [Configuration](#configuration)\n  - [Peertube](#peertube)\n  - [Youtube](#youtube)\n- [Usage](#usage)\n- [Enhanced use of NFO](#enhanced-use-of-nfo)\n- [Strict check options](#strict-check-options)\n- [Features](#features)\n- [Compatibility](#compatibility)\n- [Inspirations](#inspirations)\n- [Contributors](#contributors)\n\n## Installation and upgrade\n\n### From pip\n\nSimply install with\n```sh\npip install prismedia\n```\n\nUpgrade with\n```sh\npip install --upgrade prismedia\n```\n\n### From source\n\nGet the source:\n```sh\ngit clone https://git.lecygnenoir.info/LecygneNoir/prismedia.git prismedia\n```\n\nYou may use pip to install requirements: `pip install -r requirements.txt` if you want to use the script directly.\n(**note:** requirements are generated via `poetry export -f requirements.txt`)\n\nOtherwise, you can use [poetry](https://python-poetry.org), which create a virtualenv for the project directly\n(Or use the existing virtualenv if one is activated)\n\n```sh\npoetry install\n```\n\n\n## Configuration\n\nGenerate configuration files by running `prismedia-init`.\n\nThen, edit them to fill your credential as explained below.\n\n### Peertube\nConfiguration is in **peertube_secret** file.\nYou need your usual credentials and Peertube instance URL, in addition with API client_id and client_secret.\n\nYou can get client_id and client_secret by logging in your peertube instance and reaching the URL:\nhttps://domain.example/api/v1/oauth-clients/local\n\n*Alternatively, you can set ``OAUTHLIB_INSECURE_TRANSPORT`` to 1 if you do not use https (not recommended)*\n\n### Youtube\nConfiguration is in **youtube_secret.json** file.\nYoutube uses combination of oauth and API access to identify.\n\n**Credentials**\nThe first time you connect, prismedia will open your browser to ask you to authenticate to\nYoutube and allow the app to use your Youtube channel.\n**It is here you choose which channel you will upload to**.\nOnce authenticated, the token is stored inside the file `.youtube_credentials.json`.\nPrismedia will try to use this file at each launch, and re-ask for authentication if it does not exist.\n\n**Oauth**:\nThe default youtube_secret.json should allow you to upload some videos.\nIf you plan a larger usage, please consider creating your own youtube_secret file:\n\n - Go to the [Google console](https://console.developers.google.com/).\n - Create project.\n - Side menu: APIs & auth -> APIs\n - Top menu: Enabled API(s): Enable all Youtube APIs.\n - Side menu: APIs & auth -> Credentials.\n - Create a Client ID: Add credentials -> OAuth 2.0 Client ID -> Other -> Name: prismedia1 -> Create -> OK\n - Download JSON: Under the section "OAuth 2.0 client IDs". Save the file to your local system.\n - Save this JSON as your youtube_secret.json file.\n\n## Usage\nSupport only mp4 for cross compatibility between Youtube and Peertube.\n**Note that all options may be specified in a NFO file!** (see [Enhanced NFO](#enhanced-use-of-nfo))\n\nHere are some demonstration of main usage:\n\nUpload a video:\n```sh\nprismedia --file="yourvideo.mp4"\n```\n\nSpecify description and tags:\n```sh\nprismedia --file="yourvideo.mp4" -d "My supa description" -t "tag1,tag2,foo"\n```\n\nProvide a thumbnail:\n```sh\nprismedia --file="yourvideo.mp4" -d "Video with thumbnail" --thumbnail="/path/to/your/thumbnail.jpg"\n```\n\nPublish on Peertube only, while using a channel and a playlist, creating them if they do not exist:\n```sh\nprismedia --file="yourvideo.mp4" --platform=peertube --channel="Cooking recipes" --playlist="Cake recipes" --channelCreate --playlistCreate\n```\n\nUse a NFO file to specify your video options:\n(See [Enhanced NFO](#enhanced-use-of-nfo) for more precise example)\n```sh\nprismedia --file="yourvideo.mp4" --nfo /path/to/your/nfo.txt\n```\n\nUse some credits to show some activity for you apikey so the platform know it is used and would not put your quota to 0 (only Youtube currently).\n\nTo prevent Youtube from inactivating your apikey after 90days of inactivity it is recommended to launch this command automatically from a script around once a month. It will mwke a call to use a few credits from your daily quota.\nOn Linux and MacOS, you can use cron, on Windows the "Task Scheduler".\n```sh\nprismedia --hearthbeat\n```\n\nTake a look at all available options with `--help`!\n```sh\nprismedia --help\n```\n\n## Enhanced use of NFO\nSince Prismedia v0.9.0, the NFO system has been improved to allow hierarchical loading.\nFirst, **if you already used nfo**, either with `--nfo` or by using `videoname.txt`, nothing changes :-)\n\nBut you are now able to use a more flexible NFO system, by using priorities. This allows you to set some defaults to avoid recreating a full nfo for each video\n\nBasically, Prismedia will now load options in this order, using the last value found in case of conflict:\n`nfo.txt < directory_name.txt < video_name.txt < command line NFO < command line argument`\n\nYou\'ll find a complete set of samples in the [prismedia/samples](prismedia/samples) directory so let\'s take it as an example:\n```sh\n$ tree Recipes/\nRecipes/\n├── cli_nfo.txt\n├── nfo.txt\n├── samples.txt\n├── yourvideo1.mp4\n├── yourvideo1.txt\n├── yourvideo1.jpg\n├── yourvideo2.mp4\n└── yourvideo2.txt\n```\n\nBy using\n```sh\nprismedia --file=/path/to/Recipes/yourvideo1.mp4 --nfo=/path/to/Recipes/cli_nfo.txt --cca\n```\n\nPrismedia will:\n- look for options in `nfo.txt`\n- look for options in `samples.txt` (from directory name) and erase any previous conflicting options\n- look for options in `yourvideo1.txt` (from video name) and erase any previous conflicting options\n- look for options in `cli_nfo.txt` (from the `--nfo` in command line) and erase any previous conflicting options\n- erase any previous option regarding CCA as it\'s specified in cli with `--cca`\n- take `yourvideo1.jpg` as thumbnail if no other files has been specified in previous NFO\n\nIn other word, Prismedia will use option given in cli, then look for option in cli_nfo.txt, then complete with video_name.txt, then directory_name.txt, and finally complete with nfo.txt\n\nIt allows to specify more easily default options for an entire set of video, directory, playlist and so on.\n\n## Strict check options\nSince prismedia v0.10.0, a bunch of special options have been added to force the presence of parameters before uploading.\nStrict options allow you to force some option to be present when uploading a video. It\'s useful to be sure you do not\nforget something when uploading a video, for example if you use multiples NFO. You may force the presence of description,\ntags, thumbnail, ...\nAll strict option are optionals and are provided only to avoid errors when uploading :-)\nAll strict options can be specified in NFO directly, the only strict option mandatory on cli is --withNFO\nAll strict options are off by default.\n\nAvailable strict options:\n  - --withNFO         Prevent the upload without a NFO, either specified via cli or found in the directory\n  - --withThumbnail       Prevent the upload without a thumbnail\n  - --withName        Prevent the upload if no name are found\n  - --withDescription     Prevent the upload without description\n  - --withTags        Prevent the upload without tags\n  - --withPlaylist    Prevent the upload if no playlist\n  - --withPublishAt    Prevent the upload if no schedule\n  - --withPlatform    Prevent the upload if at least one platform is not specified\n  - --withCategory    Prevent the upload if no category\n  - --withLanguage    Prevent upload if no language\n  - --withChannel     Prevent upload if no channel\n\n## Features\n\n- [x] Youtube upload\n- [x] Peertube upload\n- Support of videos parameters (description, tags, category, licence, ...)\n  - [x] description\n  - [x] tags (no more than 30 characters per tag as Peertube does not support it)\n  - [x] categories\n  - [x] license: cca or not (Youtube only as Peertube uses Attribution by design)\n  - [x] privacy (between public, unlisted or private)\n  - [x] enabling/disabling comment (Peertube only as Youtube API does not support it)\n  - [x] nsfw (Peertube only as Youtube API does not support it)\n  - [x] set default language\n  - [x] thumbnail\n  - [x] multiple lines description (see [issue 4](https://git.lecygnenoir.info/LecygneNoir/prismedia/issues/4))\n  - [x] add videos to playlist\n  - [x] create playlist\n  - [x] schedule your video with publishAt\n  - [x] combine channel and playlist (Peertube only as channel is Peertube feature). See [issue 40](https://git.lecygnenoir.info/LecygneNoir/prismedia/issues/40) for detailed usage.\n- [x] Use a config file (NFO) file to retrieve videos arguments\n- [x] Allow choosing peertube or youtube upload (to retry a failed upload for example)\n- [x] Usable on Desktop (Linux and/or Windows and/or MacOS)\n- [x] Different schedules on platforms to prepare preview\n- [x] Possibility to force the presence of upload options\n- [ ] Copy and forget, eg possibility to copy video in a directory, and prismedia uploads itself: [Work in progress](https://git.lecygnenoir.info/Zykino/prismedia-autoupload) thanks to @Zykino 🎉 (Discussions in [issue 27](https://git.lecygnenoir.info/LecygneNoir/prismedia/issues/27))\n- [ ] A usable graphical interface\n\n## Compatibility\n\n - If you still use python2, use the version 0.7.1 (no more updated)\n - If you use peertube before 1.0.0-beta4, use the version inside tag 1.0.0-beta3\n\n## Inspirations\nInspired by peeror (First peertube mirror by Rigelk) and [youtube-upload](https://github.com/tokland/youtube-upload)\n\n## Contributors\nThanks to: @LecygneNoir, @Zykino, @meewan, @rigelk 😘\n',
    'author': 'LecygneNoir',
    'author_email': 'git@lecygnenoir.info',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://git.lecygnenoir.info/LecygneNoir/prismedia',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)



#!/usr/bin/env python
# coding: utf-8

"""
prismedia - tool to upload videos to Peertube and Youtube

Usage:
  prismedia --file=<FILE> [options]
  prismedia -f <FILE> --tags=STRING [options]
  prismedia --hearthbeat
  prismedia -h | --help
  prismedia --version

Options:
  -f, --file=STRING Path to the video file to upload. This is the only mandatory option.
  --name=NAME  Name of the video to upload. (default to video filename)
  -d, --description=STRING  Description of the video. (default: default description)
  -t, --tags=STRING  Tags for the video. comma separated.
                     WARN: tags with punctuation (!, ', ", ?, ...)
                           are not supported by Mastodon to be published from Peertube
  -c, --category=STRING  Category for the videos, see below. (default: Films)
  --cca  License should be CreativeCommon Attribution (affects Youtube upload only)
  -p, --privacy=STRING  Choose between public, unlisted or private. (default: private)
  --disable-comments  Disable comments (Peertube only as YT API does not support) (default: comments are enabled)
  --nsfw  Set the video as No Safe For Work (Peertube only as YT API does not support) (default: video is safe)
  --nfo=STRING  Configure a specific nfo file to set options for the video.
                By default Prismedia search a .txt based on the video name and will
                decode the file as UTF-8 (so make sure your nfo file is UTF-8 encoded)
                See nfo_example.txt for more details
  --platform=STRING  List of platform(s) to upload to, comma separated.
                     Supported platforms are youtube and peertube (default is both)
  --language=STRING  Specify the default language for video. See below for supported language. (default is English)
  --publishAt=DATE  Publish the video at the given DATE using local server timezone.
                    DATE should be on the form YYYY-MM-DDThh:mm:ss eg: 2018-03-12T19:00:00
                    DATE should be in the future
  --peertubeAt=DATE
  --youtubeAt=DATE  Override publishAt for the corresponding platform. Allow to create preview on specific platform
  --originalDate=DATE  Configure the video as initially recorded at DATE
                        DATE should be on the form YYYY-MM-DDThh:mm:ss eg: 2018-03-12T19:00:00
                        DATE should be in the past
  --auto-originalDate     Automatically use the file modification time as original date
  --thumbnail=STRING    Path to a file to use as a thumbnail for the video.
                        By default, prismedia search for an image based on video name followed by .jpg, .jpeg or .png
  --channel=STRING Set the channel to use for the video (Peertube only)
                    If the channel is not found, spawn an error except if --channelCreate is set.
  --channelCreate  Create the channel if not exists. (Peertube only, default do not create)
                   Only relevant if --channel is set.
  --playlist=STRING Set the playlist to use for the video.
                    If the playlist is not found, spawn an error except if --playlistCreate is set.
  --playlistCreate  Create the playlist if not exists. (default do not create)
                    Only relevant if --playlist is set.
  --progress=STRING Set the progress bar view, one of percentage, bigFile (MB), accurate (KB).

  --hearthbeat  Use some credits to show some activity for you apikey so the platform know it is used and would not put your quota to 0 (only Youtube currently)

  -h --help  Show this help.
  --version  Show version.

Logging options
  -q --quiet        Suppress any log except Critical (alias for --log=critical).
  --log=STRING      Log level, between debug, info, warning, error, critical. Ignored if --quiet is set (default to info)
  -u --url-only     Display generated URL after upload directly on stdout, implies --quiet
  --batch           Display generated URL after upload with platform information for easier parsing. Implies --quiet
                    Be careful --batch and --url-only are mutually exclusives.

Strict options:
  Strict options allow you to force some option to be present when uploading a video. It's useful to be sure you do not
  forget something when uploading a video, for example if you use multiples NFO. You may force the presence of description,
  tags, thumbnail, ...
  All strict option are optionals and are provided only to avoid errors when uploading :-)
  All strict options can be specified in NFO directly, the only strict option mandatory on cli is --withNFO
  All strict options are off by default

  --withNFO         Prevent the upload without a NFO, either specified via cli or found in the directory
  --withThumbnail       Prevent the upload without a thumbnail
  --withName        Prevent the upload if no name are found
  --withDescription     Prevent the upload without description
  --withTags        Prevent the upload without tags
  --withPlaylist    Prevent the upload if no playlist
  --withPublishAt    Prevent the upload if no schedule
  --withOriginalDate    Prevent the upload if no original date configured
  --withPlatform    Prevent the upload if at least one platform is not specified
  --withCategory    Prevent the upload if no category
  --withLanguage    Prevent upload if no language
  --withChannel     Prevent upload if no channel

Categories:
  Category is the type of video you upload. Default is films.
  Here are available categories from Peertube and Youtube:
    music, films, vehicles,
    sports, travels, gaming, people,
    comedy, entertainment, news,
    how to, education, activism, science & technology,
    science, technology, animals

Languages:
  Language of the video (audio track), choose one. Default is English
  Here are available languages from Peertube and Youtube:
    Arabic, English, French, German, Hindi, Italian,
    Japanese, Korean, Mandarin, Portuguese, Punjabi, Russian, Spanish

"""
import sys
if sys.version_info[0] < 3:
    raise Exception("Python 3 or a more recent version is required.")

import os
import datetime
import logging
logger = logging.getLogger('Prismedia')

from docopt import docopt

from . import yt_upload
from . import pt_upload
from . import utils

try:
    # noinspection PyUnresolvedReferences
    from schema import Schema, And, Or, Optional, SchemaError, Hook, Use
except ImportError:
    logger.critical('This program requires that the `schema` data-validation library'
                  ' is installed: \n'
                  'see https://github.com/halst/schema\n')
    exit(1)

VERSION = "prismedia v0.12.2"

VALID_PRIVACY_STATUSES = ('public', 'private', 'unlisted')
VALID_CATEGORIES = (
    "music", "films", "vehicles",
    "sports", "travels", "gaming", "people",
    "comedy", "entertainment", "news",
    "how to", "education", "activism", "science & technology",
    "science", "technology", "animals"
)
VALID_PLATFORM = ('youtube', 'peertube', 'none')
VALID_LANGUAGES = ('arabic', 'english', 'french',
                   'german', 'hindi', 'italian',
                   'japanese', 'korean', 'mandarin',
                   'portuguese', 'punjabi', 'russian', 'spanish')
VALID_PROGRESS = ('percentage', 'bigfile', 'accurate')


def validateCategory(category):
    if category.lower() in VALID_CATEGORIES:
        return True
    else:
        return False


def validatePrivacy(privacy):
    if privacy.lower() in VALID_PRIVACY_STATUSES:
        return True
    else:
        return False


def validatePlatform(platform):
    for plfrm in platform.split(','):
        if plfrm.lower().replace(" ", "") not in VALID_PLATFORM:
            return False

    return True


def validateLanguage(language):
    if language.lower() in VALID_LANGUAGES:
        return True
    else:
        return False


def validatePublishDate(publishDate):
    # Check date format and if date is future
    try:
        now = datetime.datetime.now()
        publishAt = datetime.datetime.strptime(publishDate, '%Y-%m-%dT%H:%M:%S')
        if now >= publishAt:
            return False
    except ValueError:
        return False
    return True


def validateOriginalDate(originalDate):
    # Check date format and if date is past
    try:
        now = datetime.datetime.now()
        originalDate = datetime.datetime.strptime(originalDate, '%Y-%m-%dT%H:%M:%S')
        if now <= originalDate:
            return False
    except ValueError:
        return False
    return True


def validateLogLevel(loglevel):
    numeric_level = getattr(logging, loglevel, None)
    if not isinstance(numeric_level, int):
        return False
    return True


def validateProgress(progress):
    for prgs in progress.split(','):
        if prgs.lower().replace(" ", "") not in VALID_PROGRESS:
            return False

    return True


def _optionnalOrStrict(key, scope, error):
    option = key.replace('-', '')
    option = option[0].upper() + option[1:]
    if scope["--with" + option] is True and scope[key] is None:
        logger.critical("Prismedia: you have required the strict presence of " + key + " but none is found")
        exit(1)
    return True


def configureLogs(options):
    if options.get('--batch') and options.get('--url-only'):
        logger.critical("Prismedia: Please use either --batch OR --url-only, not both.")
        exit(1)
    # batch and url-only implies quiet
    if options.get('--batch') or options.get('--url-only'):
        options['--quiet'] = True

    for handler in logger.handlers or logger.parent.handlers:
        if options.get('--quiet'):
            # We need to set both log level in the same time
            logger.setLevel(50)
            handler.setLevel(50)
        elif options.get('--log'):
            numeric_level = getattr(logging, options["--log"], None)
            # We need to set both log level in the same time
            logger.setLevel(numeric_level)
            handler.setLevel(numeric_level)
        elif options.get('--debug'):
            # Deprecated,
            logger.setLevel(10)
            handler.setLevel(10)


def configureStdoutLogs():
    logger_stdout = logging.getLogger('stdoutlogs')
    logger_stdout.setLevel(logging.INFO)
    ch_stdout = logging.StreamHandler(stream=sys.stdout)
    ch_stdout.setLevel(logging.INFO)
    # Default stdout logs is url only
    formatter_stdout = logging.Formatter('%(message)s')
    ch_stdout.setFormatter(formatter_stdout)
    logger_stdout.addHandler(ch_stdout)

def main():
    options = docopt(__doc__, version=VERSION)

    earlyoptionSchema = Schema({
        Optional('--log'): Or(None, And(
                                str,
                                Use(str.upper),
                                validateLogLevel,
                                error="Log level not recognized")
                              ),
        Optional('--quiet', default=False): bool,
        Optional('--debug'): bool,
        Optional('--url-only', default=False): bool,
        Optional('--batch', default=False): bool,
        Optional('--withNFO', default=False): bool,
        Optional('--withThumbnail', default=False): bool,
        Optional('--withName', default=False): bool,
        Optional('--withDescription', default=False): bool,
        Optional('--withTags', default=False): bool,
        Optional('--withPlaylist', default=False): bool,
        Optional('--withPublishAt', default=False): bool,
        Optional('--withOriginalDate', default=False): bool,
        Optional('--withPlatform', default=False): bool,
        Optional('--withCategory', default=False): bool,
        Optional('--withLanguage', default=False): bool,
        Optional('--withChannel', default=False): bool,
        # This allow to return all other options for further use: https://github.com/keleshev/schema#extra-keys
        object: object
    })

    schema = Schema({
        '--file': And(str, os.path.exists, error='file does not exists, please check path'),
        # Strict option checks - at the moment Schema needs to check Hook and Optional separately #
        Hook('--name', handler=_optionnalOrStrict): object,
        Hook('--description', handler=_optionnalOrStrict): object,
        Hook('--tags', handler=_optionnalOrStrict): object,
        Hook('--category', handler=_optionnalOrStrict): object,
        Hook('--language', handler=_optionnalOrStrict): object,
        Hook('--platform', handler=_optionnalOrStrict): object,
        Hook('--publishAt', handler=_optionnalOrStrict): object,
        Hook('--originalDate', handler=_optionnalOrStrict): object,
        Hook('--thumbnail', handler=_optionnalOrStrict): object,
        Hook('--channel', handler=_optionnalOrStrict): object,
        Hook('--playlist', handler=_optionnalOrStrict): object,
        # Validate checks #
        Optional('--name'): Or(None, And(
                                str,
                                lambda x: not x.isdigit(),
                                error="The video name should be a string")
                               ),
        Optional('--description'): Or(None, And(
                                        str,
                                        lambda x: not x.isdigit(),
                                        error="The video description should be a string")
                                      ),
        Optional('--tags'): Or(None, And(
                                    str,
                                    lambda x: not x.isdigit(),
                                    error="Tags should be a string")
                               ),
        Optional('--category'): Or(None, And(
                                    str,
                                    validateCategory,
                                    error="Category not recognized, please see --help")
                                   ),
        Optional('--language'): Or(None, And(
                                    str,
                                    validateLanguage,
                                    error="Language not recognized, please see --help")
                                   ),
        Optional('--privacy'): Or(None, And(
                                    str,
                                    validatePrivacy,
                                    error="Please use recognized privacy between public, unlisted or private")
                                  ),
        Optional('--nfo'): Or(None, str),
        Optional('--platform'): Or(None, And(str, validatePlatform, error="Sorry, upload platform not supported")),
        Optional('--publishAt'): Or(None, And(
                                    str,
                                    validatePublishDate,
                                    error="Publish Date should be the form YYYY-MM-DDThh:mm:ss and has to be in the future")
                                    ),
        Optional('--peertubeAt'): Or(None, And(
                                    str,
                                    validatePublishDate,
                                    error="Publish Date should be the form YYYY-MM-DDThh:mm:ss and has to be in the future")
                                    ),
        Optional('--youtubeAt'): Or(None, And(
                                    str,
                                    validatePublishDate,
                                    error="Publish Date should be the form YYYY-MM-DDThh:mm:ss and has to be in the future")
                                    ),
        Optional('--originalDate'): Or(None, And(
                                    str,
                                    validateOriginalDate,
                                    error="Original date should be the form YYYY-MM-DDThh:mm:ss and has to be in the past")
                                    ),
        Optional('--auto-originalDate'): bool,
        Optional('--cca'): bool,
        Optional('--disable-comments'): bool,
        Optional('--nsfw'): bool,
        Optional('--thumbnail'): Or(None, And(
                                    str, os.path.exists, error='Thumbnail does not exists, please check the path.'),
                                    ),
        Optional('--channel'): Or(None, str),
        Optional('--channelCreate'): bool,
        Optional('--playlist'): Or(None, str),
        Optional('--playlistCreate'): bool,
        Optional('--progress'): Or(None, And(str, validateProgress, error="Sorry, progress visualisation not supported")),
        '--hearthbeat': bool,
        '--help': bool,
        '--version': bool,
        # This allow to return all other options for further use: https://github.com/keleshev/schema#extra-keys
        object: object
    })

    if options.get('--hearthbeat'):
        yt_upload.hearthbeat()
        exit(0)

    # We need to validate early options first as withNFO and logs options should be prioritized
    try:
        options = earlyoptionSchema.validate(options)
        configureLogs(options)
    except SchemaError as e:
        logger.critical(e)
        exit(1)

    if options.get('--url-only') or options.get('--batch'):
        configureStdoutLogs()

    options = utils.parseNFO(options)

    # If after loading NFO we still has no original date and --auto-originalDate is enabled,
    # then we need to search from the file
    # We need to do that before the strict validation in case --withOriginalDate is enabled
    if not options.get('--originalDate') and options.get('--auto-originalDate'):
        options['--originalDate'] = utils.searchOriginalDate(options)

    # Once NFO are loaded, we need to revalidate strict options in case some were in NFO
    try:
        options = earlyoptionSchema.validate(options)
    except SchemaError as e:
        logger.critical(e)
        exit(1)

    if not options.get('--thumbnail'):
        options = utils.searchThumbnail(options)

    try:
        options = schema.validate(options)
    except SchemaError as e:
        logger.critical(e)
        exit(1)

    logger.debug("Python " + sys.version)
    logger.debug(options)

    if options.get('--platform') is None or "peertube" in options.get('--platform'):
        pt_upload.run(options)
    if options.get('--platform') is None or "youtube" in options.get('--platform'):
        yt_upload.run(options)


if __name__ == '__main__':
    logger.warning("DEPRECATION: use 'python -m prismedia', not 'python -m prismedia.upload'")
    main()
